function demo2D04
% demo2D04
%
% Trains a 2D diffeomorphic using the 2D HeLa dataset provided and the
% synthesize images from the model
%
% by the Murphy Lab at http://murphylab.web.cmu.edu/data/
%
% Author: Xiongtao Ruan (xruan@andrew.cmu.edu)
%
% Copyright (C) 2013-2015 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


directory = what(mfilename);

if isfield( directory, 'path' )
    directory = directory.path;
end
cd( directory );

clc;

%% Training a 2D diffeomorphic model from Hela 2D image datasets
pattern = 'lam';

%downloading images
tarball = [ 'hela2d-' pattern '.tgz' ];
url = 'http://murphylab.web.cmu.edu/data/Hela/2D/';

if ~exist( [ pwd filesep tarball ] )
    disp('Downloading dataset');
    urlwrite( [ url tarball] , tarball );
    disp('Extracting files');
    untar( [ pwd filesep tarball ] );
else
    disp('Pattern found on disk. Extracting files from existing tarball');
    disp('Extracting files');
    system(['tar -xvf ./' tarball] );
end

% set image paths of dna, cell and masks
dna = [ directory filesep 'LAM/orgdna' filesep 'cell*2.tif' ];
cellm = [ directory filesep 'LAM/orgcell' filesep 'cell*2.tif' ];

param.masks = [ directory filesep 'LAM/crop' filesep 'cell*2.tif' ];


%set model resolution 
param.model.resolution = [ 0.49, 0.49 ];

%set model name
param.model.name = mfilename;

%set nuclei and cell model name
param.nucleus.name = 'LAMP_2d';
param.cell.model = 'LAMP_2d';

%set the dimensionality of the model
dimensionality = '2D';

%set the model id, nuclei model id and cell model id
try
   [status,result] = system( 'uuidgen' ); 
   param.model.id = result(1:end-1);
catch
   param.model.id = num2str(now);
end

try
   [status,result] = system( 'uuidgen' ); 
   param.nucleus.id = result(1:end-1);
catch
   param.nucleus.id = num2str(now);
end

try
   [status,result] = system( 'uuidgen' ); 
   param.cell.id = result(1:end-1);
catch
   param.cell.id = num2str(now);
end

%set model filename
param.model.filename = [ lower(pattern) '.xml' ];

%set model type
param.nucleus.type = 'diffeomorphic';

param.cell.type = 'diffeomorphic';

% verbose                   (optional) Displays messages to screen. The default is true.
param.verbose = true;

% debug                     (optional) Reports errors and warnings. Default is false.
param.debug = true;

% train.flag                (optional) Selects what model is going to be trained ('nuclear',
%                           'framework', or 'all'). Default is 'all'
param.train.flag = 'framework';

%documentation
param.documentation.description = 'This model has been trained using demo2D04 from CellOrganizer';

%model resolution
param.model.downsampling = [5,5];

%this is the resolution of the datasets
param.model.resolution = [0.049, 0.049];

%set alignment method
param.model.diffeomorphic.com_align = 'nuc';

%train the model
success = img2slml( dimensionality, dna, cellm,[], param );

load('lam.mat');

%show the shape space and save the image
img = showShapeSpace(model);

imwrite(img, ['demo2D04ShapeSpace.png'], 'png')

%% Synthesize images from the model above

%reset parameter for synthesis
param = [];

%set directory to store images
outputDirectory = [directory, filesep, 'synthesized_images'];
% check if the directory exists
if ~exist( outputDirectory, 'dir' )
  mkdir( outputDirectory );
end
 
param.targetDirectory = outputDirectory;
%set the prefix of the images 
param.prefix = 'demo2D04_';
%set the number of images that would be synthesized
param.numberOfSynthesizedImages = 1;
%set compression method for images
param.compression = 'lzw';
param.microscope = 'none';
% verbose                   (optional) Displays messages to screen. The default is true.
param.verbose = true;

% debug                     (optional) Reports errors and warnings. Default is false.
param.debug = true;

%synthesize method
param.synthesis = 'framework';
param.synthesis.synthesis = 'framework';

%iterations for sampling
param.synthesis.diffeomorphic.maximum_iterations = 40;

model_name = [ directory filesep 'lam.mat' ];

%synthesize images 
slml2img( {model_name}, param );


end



